package com.huguesjohnson.jail.jailBreak;

/*
jailBreak - Package to launch Java Artificial Intelligence Library components
JailBreak.java - entry point for the program 
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.Boolean;
import java.lang.ClassNotFoundException;
import java.lang.Double;
import java.lang.Exception;
import java.lang.Object;
import java.lang.StackTraceElement;
import java.lang.String;
import java.lang.StringBuffer;
import java.lang.System;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import com.huguesjohnson.DataFile;
import com.huguesjohnson.LaunchBrowser;
import com.huguesjohnson.jail.jailBreak.JailBreakMainFrame;
import com.huguesjohnson.jail.jailBreak.JailBreakUIConstants;
import com.huguesjohnson.jail.jailBreak.event.JailBreakActionEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakOpenFileEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakTreeSelectionEvent;
import com.huguesjohnson.jail.jailBreak.fileFilter.BackPropagationNetworkFilter;
import com.huguesjohnson.jail.jailBreak.fileFilter.NeuralNetworkDataFileFilter;
import com.huguesjohnson.jail.jailBreak.fileFilter.NeuralNetworkTrainingFileFilter;
import com.huguesjohnson.jail.jailBreak.wizard.JailBreakNewBackPropagationNetworkWizard;
import com.huguesjohnson.jail.jailBreak.wizard.JailBreakStartupWizard;
import com.huguesjohnson.jail.neuralNetwork.BackPropagationNetwork;
import com.huguesjohnson.jail.neuralNetwork.Neuron;
import com.huguesjohnson.jail.neuralNetwork.NeuralNetwork;
import com.huguesjohnson.jail.neuralNetwork.Layer;
import com.huguesjohnson.ui.SplashScreen;

/** class JailBreak
 * @author Hugues Johnson
 */
public class JailBreak extends Object{
    /** change this to false before building for distribution*/
    private final static boolean _DEV_MODE_=false;
    
    /** reference to the main window */
    private JailBreakMainFrame mainWindow;
    
    /** the path to the folder containing images for current theme */
    private String resourcePath;

    /** the path to the folder containing help files */
    private String helpPath;    
    
    /** the path to the folder containing default images */
    private String defaultResourcePath;
    
    /** the last directory that where a file was saved or loaded */
    private String lastPath;
    
    /** the current file that's being worked on */
    private String currentFile;

    /** whether or not to write debug messages */
    private boolean debugMessages;
    
    /** begin references to objects that can be viewed in this application.. there's certainly a better way to do this but I haven't thought of it yet */
    /** what object is currently being viewed */
    private int currentView; 
    /** currently viewing backpropagation neural network */    
    private final static int VIEW_BPNN=0; 
    /** reference to a backpropagation neural network */    
    private BackPropagationNetwork bpNetwork;
    /** end references to objects that can be viewed in this application */
    
    /** constructor sets the resource path and creates the window
     */
    public JailBreak(){
        /* debug messages off by default */
        this.debugMessages=false;
        if(_DEV_MODE_){
            this.resourcePath="c:\\Projects\\themes\\Default\\";
            this.defaultResourcePath="c:\\Projects\\themes\\Default\\";
            this.lastPath="c:\\Projects\\";
            this.helpPath="c:\\Projects\\docs\\jail\\";
        } else{
            String userDir=new String(System.getProperty("user.dir"));
            this.resourcePath=new String(userDir+File.separator+"themes"+File.separator+"default"+File.separator);
            this.defaultResourcePath=new String(userDir+File.separator+"themes"+File.separator+"default"+File.separator);
            this.lastPath=new String(userDir+File.separator);
            this.helpPath=new String(userDir+File.separator+"docs"+File.separator+"jail"+File.separator);
        }
        this.createWindow();
        /* make sure window has been created successfully */
        if(this.mainWindow==null){
            System.err.println(JailBreakUIConstants.ERROR_UNABLE_TO_LOAD);
            System.exit(0);
        }
        SplashScreen splash=new SplashScreen(this.mainWindow,this.resourcePath+"splash_jailbreak.gif");
        /* finish loading main window */
        this.initWindow();
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        /* create startup dialog */
        JailBreakStartupWizard startupWizard=new JailBreakStartupWizard(this.mainWindow,this.resourcePath,this.lastPath);
        /* dispose splash screen */
        splash.dispose();
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        startupWizard.show();
        /* handle the event returned by the startup dialog */
        this.eventHandler(startupWizard.getExitActionEvent());
    }

    /** resets the current view
     * @param newView which view to switch to
     */
    private final void resetView(int newView){
        this.currentView=newView;
        switch(newView){
            case VIEW_BPNN: {
                this.mainWindow.updateWindow(this.bpNetwork);
                break;
            }
            default: {
                break;
            }
        }
    }
    
    /** createWindow
     */
    private void createWindow(){
        mainWindow=new JailBreakMainFrame(this.resourcePath,this.defaultResourcePath){
            public void sendEvent(JailBreakEvent event){
                eventHandler(event);
            }
            public void closeWindow(){
                unloadWindow(true);
            }
        };
    }

    /** initWindow intializes mainWindow and shows it
     */
    private final void initWindow(){
        //this.mainWindow.pack();
        this.mainWindow.setVisible(true);
    }    
    
     /** unloadWindow receives the windowClosing event from the main window
      * @param exitSystem flag to indicate whether or not to call System.exit()
      */
    private final void unloadWindow(boolean exitSystem){
        //this.saveSettings();
        this.mainWindow.dispose();
        /* check status before deciding whether or not to exit */
        if(exitSystem){
            System.exit(0);
        }
    }
    
    /** receives all GUI events passed to the main window
     * @param event the event received by the main window
     */
    private final void eventHandler(JailBreakEvent event){
        this.writeDebugMessage("Received event: "+event.toString());
        this.writeDebugMessage("Event class name: "+event.getClass().getName());
        /* what type of event was received */
        if(event instanceof JailBreakActionEvent){
            this.handleActionEvent((JailBreakActionEvent)event);
        } else if(event instanceof JailBreakOpenFileEvent){
            this.openSerializedObject(((JailBreakOpenFileEvent)event).getFileName(),((JailBreakOpenFileEvent)event).getFilter());
        } else if(event instanceof JailBreakTreeSelectionEvent){
            this.updateUI(((JailBreakTreeSelectionEvent)event).getUserObject());
        }
    }    

    /** updates the user interface based on the type of object received
     * @param object the object to update the gui based on
     */
    private void updateUI(Object object){
        if(object instanceof NeuralNetwork){
            this.mainWindow.updateWindow((NeuralNetwork)object);
        } else if(object instanceof Layer){
            this.mainWindow.updateWindow((Layer)object);
        } else if(object instanceof Neuron){
            this.mainWindow.updateWindow((Neuron)object);
        }
    }
    
    /** handles action events 
     * @param event the action event to handle
     */
    private final void handleActionEvent(JailBreakActionEvent event){
        String actionCommand=event.getActionCommand();
        String parameter=event.getParameter();
        if(actionCommand==JailBreakUIConstants.ACTION_EXIT){
            this.unloadWindow(true);
        } else if(actionCommand==JailBreakUIConstants.ACTION_CHANGE_STYLE){
            try{
                UIManager.setLookAndFeel(parameter);
                SwingUtilities.updateComponentTreeUI(this.mainWindow);
            } catch(Exception x){
                this.errorDialog(x);
            }
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_TOOLBAR){
            this.mainWindow.setToolBarVisible(Boolean.valueOf(parameter).booleanValue());
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_MENU){
            this.mainWindow.setMenuBarVisible(Boolean.valueOf(parameter).booleanValue());
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_STATUS){
            this.mainWindow.setStatusBarVisible(Boolean.valueOf(parameter).booleanValue());
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_BROWSER){
            this.mainWindow.setObjectBrowserVisible(Boolean.valueOf(parameter).booleanValue());
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_NODE){
            this.mainWindow.setObjectViewerVisible(Boolean.valueOf(parameter).booleanValue());
        } else if(actionCommand==JailBreakUIConstants.ACTION_VIEW_DEBUG){
            this.debugMessages=Boolean.valueOf(parameter).booleanValue();
            this.mainWindow.setDebugMessageFrameVisible(this.debugMessages);
        } else if(actionCommand==JailBreakUIConstants.ACTION_HELP_ABOUT){
            this.showAboutDialog();
        } else if(actionCommand==JailBreakUIConstants.ACTION_HELP_INDEX){
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            this.writeDebugMessage((new LaunchBrowser()).launch(this.helpPath+"index.html",true));
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        } else if(actionCommand==JailBreakUIConstants.ACTION_HELP_API){
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            this.writeDebugMessage((new LaunchBrowser()).launch(this.helpPath+"api"+File.separator+"index.html",true));
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        } else if(actionCommand==JailBreakUIConstants.ACTION_NEW_BPNN){
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            JailBreakNewBackPropagationNetworkWizard newBpnnWizard=new JailBreakNewBackPropagationNetworkWizard(this.mainWindow,this.resourcePath,this.defaultResourcePath);
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            newBpnnWizard.show();
            /* check if 'ok' was selected */
            if(newBpnnWizard.getExitActionEvent().getActionCommand()==JailBreakUIConstants.ACTION_OK){
                this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                int inputLayerSize=newBpnnWizard.getInputLayerSize();
                int hiddenLayerSize=newBpnnWizard.getHiddenLayerSize();
                int outputLayerSize=newBpnnWizard.getOutputLayerSize();
                int inputSize=newBpnnWizard.getInputSize();
                double punishRate=newBpnnWizard.getPunishRate();
                double rewardRate=newBpnnWizard.getRewardRate();
                double errorThreshold=newBpnnWizard.getErrorThreshold();
                double minWeight=newBpnnWizard.getMinWeight();
                double maxWeight=newBpnnWizard.getMaxWeight();
                this.bpNetwork=new BackPropagationNetwork(inputLayerSize,hiddenLayerSize,outputLayerSize,inputSize,punishRate,rewardRate,errorThreshold,minWeight,maxWeight);
                /* set default input */
                double[] defaultInput=new double[inputSize];
                for(int index=0;index<inputSize;index++){
                    defaultInput[index]=0.0D;
                }
                bpNetwork.setInput(defaultInput);
                bpNetwork.run();
                this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                this.resetView(this.VIEW_BPNN);
            }
        } else if(actionCommand==JailBreakUIConstants.ACTION_OPEN){
            this.openSerializedObject();
        } else if(actionCommand==JailBreakUIConstants.ACTION_SAVE){
            if(this.currentFile==null){ //create a function to lookup if the current file exists instead of just checking for null
                this.handleActionEvent(new JailBreakActionEvent(JailBreakUIConstants.ACTION_SAVE_AS));
            } else{
                this.serializeCurrentObject();
            }
        } else if(actionCommand==JailBreakUIConstants.ACTION_SAVE_AS){
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            /* open a save as dialog */
            JFileChooser fc=new JFileChooser(this.lastPath);
            fc.setAcceptAllFileFilterUsed(false);
            switch(this.currentView){
                case VIEW_BPNN: {
                    /* add BackPropagationNetwork filter  */
                    fc.addChoosableFileFilter(new BackPropagationNetworkFilter());
                    break;
                }
                default: {
                    /* no object is currently open, in theory this should be unreachable */
                    break;
                }
            }
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            int returnValue=fc.showSaveDialog(mainWindow);
            if(returnValue==fc.APPROVE_OPTION){
                /* save the path */
                this.lastPath=fc.getCurrentDirectory().getPath();
                /* save the name of the file selected */
                this.currentFile=this.lastPath+File.separator+fc.getSelectedFile().getName();
                /* TODO - make sure the file ends with the extention from the filter */
                /* serialize the object */
                this.serializeCurrentObject();
            }
        } else if(actionCommand==JailBreakUIConstants.ACTION_RUN){
            this.runCurrentObject();
        } else if(actionCommand==JailBreakUIConstants.ACTION_RUN_FILE){
            this.runFile();
        } else if(actionCommand==JailBreakUIConstants.ACTION_TRAIN){
            this.trainCurrentObject();
        } else if(actionCommand==JailBreakUIConstants.ACTION_TRAIN_FILE){
            this.trainFile();
        }
    }

    /** runs the currently open object with it'an input set from a file
     */    
    private final void runFile(){
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        /* browse for a file to run */
        JFileChooser fc=new JFileChooser(this.lastPath);
        /* add appropriate filter */
        switch(this.currentView){
            case VIEW_BPNN: {
                fc.addChoosableFileFilter(new NeuralNetworkDataFileFilter());
            }
            default: {
                /* no object is currently open, in theory this should be unreachable */
            }
        }
        fc.setAcceptAllFileFilterUsed(false);
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        int returnValue=fc.showOpenDialog(mainWindow);
        if(returnValue==fc.APPROVE_OPTION){
            /* save the path */
            this.lastPath=fc.getCurrentDirectory().getPath();
            /* parse the file */
            String dataFileName=this.lastPath+File.separator+fc.getSelectedFile().getName();
            DataFile dataFile=new DataFile(dataFileName);
            while(dataFile.endOfFile()==false){
                String[] sInput=dataFile.getNextLine();
                /* convert to double.. probably should move this into the data file class */
                double[] input=new double[sInput.length];
                for(int index=0;index<sInput.length;index++){
                    input[index]=Double.valueOf(sInput[index]).doubleValue();
                }
                sInput=null;
                /* probably a better way to do all this too.. */
                switch(this.currentView){
                    case VIEW_BPNN: {
                        this.bpNetwork.setInput(input);
                        break;
                    }
                    default: {
                        /* no object is currently open, in theory this should be unreachable */
                        break;
                    }
                }
                /* now run the current object */
                this.runCurrentObject();
            }
        }
    }
    
    /** runs the currently open object with it's current input set 
     */
    private final void runCurrentObject(){
        try{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));            
            /* probably better way to do this */
            switch(this.currentView){
                case VIEW_BPNN: {
                    this.mainWindow.setProgress(JailBreakUIConstants.STATUS_RUNNING+"Neural Network..");
                    //use start instead of run once syncronization is figured out
                    //this.bpNetwork.start();
                    this.bpNetwork.run();
                    this.mainWindow.setProgress(JailBreakUIConstants.STATUS_DONE);
                    break;
                }
                default: {
                    /* no object is currently open, in theory this should be unreachable */
                    break;
                }
            }        
        } catch(Exception x){
            this.errorDialog(x);
        } finally{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
        /* update UI */
        Object treeSelection=this.mainWindow.getTreeSelection();
        this.writeDebugMessage("Selected object in tree: "+treeSelection.toString());
        this.updateUI(treeSelection);
    }

    /** trains the currently open object with it'an input set from a file
     */    
    private final void trainFile(){
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        /* browse for a file to run */
        JFileChooser fc=new JFileChooser(this.lastPath);
        /* add appropriate filter */
        switch(this.currentView){
            case VIEW_BPNN: {
                fc.addChoosableFileFilter(new NeuralNetworkTrainingFileFilter());
            }
            default: {
                /* no object is currently open, in theory this should be unreachable */
            }
        }               
        fc.setAcceptAllFileFilterUsed(false);
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        int returnValue=fc.showOpenDialog(mainWindow);
        if(returnValue==fc.APPROVE_OPTION){
            /* save the path */
            this.lastPath=fc.getCurrentDirectory().getPath();
            /* parse the file */
            String dataFileName=this.lastPath+File.separator+fc.getSelectedFile().getName();
            DataFile dataFile=new DataFile(dataFileName);
            while(dataFile.endOfFile()==false){
                String[] sInput=dataFile.getNextLine();
                /* convert to double.. probably should move this into the data file class */
                double[] input=new double[sInput.length];
                for(int index=0;index<sInput.length;index++){
                    input[index]=Double.valueOf(sInput[index]).doubleValue();
                }
                sInput=null;
                String[] sExpectedResults=dataFile.getNextLine();
                /* convert to double.. probably should move this into the data file class */
                double[] expectedResults=new double[sExpectedResults.length];
                for(int index=0;index<sExpectedResults.length;index++){
                    expectedResults[index]=Double.valueOf(sExpectedResults[index]).doubleValue();
                }
                sExpectedResults=null;
                /* probably a better way to do all this too.. */
                switch(this.currentView){
                    case VIEW_BPNN: {
                        this.bpNetwork.setInput(input);
                        this.bpNetwork.setDesiredOutput(expectedResults);
                        break;
                    }
                    default: {
                        /* no object is currently open, in theory this should be unreachable */
                        break;
                    }
                }
                /* train the current object */
                this.trainCurrentObject();
            }
        }
    }

    /** trains the currently open object with it's current input set 
     */
    private final void trainCurrentObject(){
        try{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            switch(this.currentView){
                case VIEW_BPNN: {
                    this.mainWindow.setProgress(JailBreakUIConstants.STATUS_TRAINING+"Neural Network..");
                    //have to allow user to set this
                    this.bpNetwork.train(JailBreakUIConstants.BPNN_DEFAULT_TRAINING_ITERATIONS);
                    this.mainWindow.setProgress(JailBreakUIConstants.STATUS_DONE);
                    break;
                }
                default: {
                    /* no object is currently open, in theory this should be unreachable */
                    break;
                }
            }
        } catch(Exception x){
            this.errorDialog(x);
        } finally{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
        /* update UI */
        Object treeSelection=this.mainWindow.getTreeSelection();
        this.writeDebugMessage("Selected object in tree: "+treeSelection.toString());
        this.updateUI(treeSelection);        
    }
    
    /** serializes the object that is currently open
     */
    private final void serializeCurrentObject(){
        try{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            switch(this.currentView){
                case VIEW_BPNN: {
                    (new ObjectOutputStream(new FileOutputStream(this.currentFile))).writeObject(this.bpNetwork);
                    break;
                }
                default: {
                    /* no object is currently open, in theory this should be unreachable */
                    break;
                }
            }
        } catch(IOException iox){
            this.errorDialog(iox);
            this.mainWindow.setProgress(JailBreakUIConstants.STATUS_ERROR+JailBreakUIConstants.STATUS_SAVING+this.currentFile,0);
        } finally{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }
    
    /** opens a serialized object 
     */
    private final void openSerializedObject(){
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        /* show file chooser */
        JFileChooser fc=new JFileChooser(this.lastPath);
        /* add BackPropagationNetwork filter  */
        fc.addChoosableFileFilter(new BackPropagationNetworkFilter());
        fc.setAcceptAllFileFilterUsed(false);
        this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        int returnValue=fc.showOpenDialog(mainWindow);
        if(returnValue==fc.APPROVE_OPTION){
            /* save the path */
            this.lastPath=fc.getCurrentDirectory().getPath();
            this.openSerializedObject(this.lastPath+File.separator+fc.getSelectedFile().getName(),fc.getFileFilter());
        }
    }
    
    /** opens an object that was saved to disk
     * @param fileName the name of the file to open
     * @param filter the FileFilter (type) of the file to open
     */
    private final void openSerializedObject(String fileName,FileFilter filter){
        try{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            if(filter instanceof BackPropagationNetworkFilter){
                this.bpNetwork=(BackPropagationNetwork)(new ObjectInputStream(new FileInputStream(fileName))).readObject();
                this.resetView(this.VIEW_BPNN);
            }
        } catch(IOException iox){
            this.errorDialog(iox);
        } catch(ClassNotFoundException cnfx){
            this.errorDialog(cnfx);
        } finally{
            this.mainWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
        /* no errors, set currentFile=fileName */
        this.currentFile=fileName;
    }
    
    /** errorDialog displays an error dialog and write the error to the local error stream
     * @param x the exception that occured
     * @todo design custom error dialog with scrollbox for stack trace instead of using JOptionPane
     */
    private final void errorDialog(Exception x){
        String caption=x.getClass().getName();
        /* build error message */
        StringBuffer errorMessage=new StringBuffer("The following error has occured:\n\n");
        errorMessage.append("Message\n--------------------\n");
        errorMessage.append(x.toString());
        errorMessage.append("\n\nStack Trace\n--------------------\n");
        StackTraceElement[] stackTrace=x.getStackTrace();
        int length=stackTrace.length;
        /* limit size of stack trace to prevent ludicrously large dialogs */
        if(length<=7){
            for(int index=0;index<length;index++){
                errorMessage.append(stackTrace[index].toString()+"\n");
            }
        } else{
            for(int index=0;index<3;index++){
                errorMessage.append(stackTrace[index].toString()+"\n");
            }
            errorMessage.append("[....]\n");
            for(int index=length-1;index>length-4;index--){
                errorMessage.append(stackTrace[index].toString()+"\n");
            }
        }
        /* display the dialog */
        JOptionPane.showMessageDialog(this.mainWindow,errorMessage.toString(),caption,JOptionPane.ERROR_MESSAGE);
        /* write the message to the error stream */
        x.printStackTrace();
    }

     /** displays a program information dialog in the main window
      */
     public final void showAboutDialog(){
          JOptionPane.showMessageDialog(this.mainWindow,JailBreakUIConstants.CAPTION_ABOUT_MESSAGE,JailBreakUIConstants.CAPTION_ABOUT_WINDOW,JOptionPane.INFORMATION_MESSAGE);
     }    
    
    /** writes a message to the debug frame
     * @param message the message to write
     */
    public void writeDebugMessage(String message){
        if(this.debugMessages==true){
            this.mainWindow.writeDebugMessage(message);
        }
    }         
     
    /** entry point for program
     * @param arguments the command line arguments
     */
    public static void main(String[] arguments){
        try{
            JailBreak instance=new JailBreak();
        } catch(Exception x){
            x.printStackTrace();
        }
    }

}