package com.huguesjohnson.jail.jailBreak;

/*
jailBreak - Package to launch Java Artificial Intelligence Library components
JailBreakDesktopPane.java - Desktop pane for main window
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.lang.String;
import java.lang.StringBuffer;
import javax.swing.JDesktopPane;
import com.huguesjohnson.jail.jailBreak.JailBreakDebugMessageInternalFrame;
import com.huguesjohnson.jail.jailBreak.JailBreakObjectBrowserInternalFrame;
import com.huguesjohnson.jail.jailBreak.JailBreakObjectViewerInternalFrame;
import com.huguesjohnson.jail.jailBreak.JailBreakUIConstants;
import com.huguesjohnson.jail.jailBreak.event.JailBreakEvent;
import com.huguesjohnson.jail.neuralNetwork.Layer;
import com.huguesjohnson.jail.neuralNetwork.NeuralNetwork;
import com.huguesjohnson.jail.neuralNetwork.Neuron;

/** class JailBreakDesktopPane
 * @author  Hugues Johnson
 */
public class JailBreakDesktopPane extends JDesktopPane{
    
    /** the path to the folder containing images for current theme */
    private String resourcePath;

    /** the path to the folder containing default images */
    private String defaultResourcePath;

    /** reference to the object browser (tree) */
    JailBreakObjectBrowserInternalFrame objectBrowserInternalFrame;

    /** reference to the object viewer (grid) */
    JailBreakObjectViewerInternalFrame objectViewerInternalFrame;
    
    /** reference to the debug message window */
    JailBreakDebugMessageInternalFrame debugMessageInternalFrame;
    
    /** Creates new JailBreakDesktopPane 
     * @param resourcePath path to the folder containing images for current theme 
     * @param defaultResourcePath path to the folder containing default images   
     */
    public JailBreakDesktopPane(String resourcePath,String defaultResourcePath){
        super();
        this.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        /* add the JailBreakObjectBrowserInternalFrame */
        this.objectBrowserInternalFrame=new JailBreakObjectBrowserInternalFrame(resourcePath,defaultResourcePath){ public void sendEvent(JailBreakEvent event){ receiveInternalFrameEvent(event);}};
        this.add(this.objectBrowserInternalFrame);
        this.objectBrowserInternalFrame.setVisible(true);
        this.objectBrowserInternalFrame.setLocation(JailBreakUIConstants.BROWSE_FRAME_DEFAULT_X,JailBreakUIConstants.BROWSE_FRAME_DEFAULT_Y);
        this.objectBrowserInternalFrame.setSize(JailBreakUIConstants.BROWSE_FRAME_DEFAULT_WIDTH,JailBreakUIConstants.BROWSE_FRAME_DEFAULT_HEIGHT);
        /* add the JailBreakObjectViewerInternalFrame */
        this.objectViewerInternalFrame=new JailBreakObjectViewerInternalFrame(resourcePath,defaultResourcePath){ public void sendEvent(JailBreakEvent event){ receiveInternalFrameEvent(event);}};
        this.add(this.objectViewerInternalFrame);
        this.objectViewerInternalFrame.setVisible(true);
        this.objectViewerInternalFrame.setLocation(JailBreakUIConstants.VIEW_FRAME_DEFAULT_X,JailBreakUIConstants.VIEW_FRAME_DEFAULT_Y);
        this.objectViewerInternalFrame.setSize(JailBreakUIConstants.VIEW_FRAME_DEFAULT_WIDTH,JailBreakUIConstants.VIEW_FRAME_DEFAULT_HEIGHT);
        /* add the JailBreakDebugMessageInternalFrame */
        this.debugMessageInternalFrame=new JailBreakDebugMessageInternalFrame(resourcePath,defaultResourcePath);
        this.add(this.debugMessageInternalFrame);
        this.debugMessageInternalFrame.setVisible(false);
        this.debugMessageInternalFrame.setLocation(JailBreakUIConstants.DEBUG_FRAME_DEFAULT_X,JailBreakUIConstants.DEBUG_FRAME_DEFAULT_Y);
        this.debugMessageInternalFrame.setSize(JailBreakUIConstants.DEBUG_FRAME_DEFAULT_WIDTH,JailBreakUIConstants.DEBUG_FRAME_DEFAULT_HEIGHT);
    }
    
    /** updates the desktop based on the object passed
     * @param network a neural network to display
     */
    public void updateDesktop(NeuralNetwork network){
        this.objectBrowserInternalFrame.updateFrame(network);
        this.objectViewerInternalFrame.updateFrame(network);    
    }
    
    /** updates the desktop based on the object passed
     * @param layer a layer to display
     */
    public void updateDesktop(Layer layer){
        this.objectBrowserInternalFrame.updateFrame(layer);
        this.objectViewerInternalFrame.updateFrame(layer);    
    }

    /** updates the desktop based on the object passed
     * @param neuron a neuron to display
     */    
    public void updateDesktop(Neuron neuron){
        this.objectBrowserInternalFrame.updateFrame(neuron);
        this.objectViewerInternalFrame.updateFrame(neuron);    
    }
    
    /** writes a message to the debug frame
     * @param message the message to append to the debug frame
     */
    public void writeDebugMessage(String message){
        this.debugMessageInternalFrame.write(message);
    }    
    
    /** shows or hides the object browser (tree)
     * @param flag true to make the component visible; false to make it invisible 
     */
    public final void setObjectBrowserVisible(boolean flag){
        this.objectBrowserInternalFrame.setVisible(flag);
    }    

    /** shows or hides the object browser (tree)
     * @param flag true to make the component visible; false to make it invisible 
     */
    public final void setObjectViewerVisible(boolean flag){
        this.objectViewerInternalFrame.setVisible(flag);
    }    
    
    /** shows or hides debug messages
     * @param flag true to make the component visible; false to make it invisible 
     */
    public final void setDebugMessageFrameVisible(boolean flag){
        this.debugMessageInternalFrame.setVisible(flag);
    }    
    
    /** override when creating a new instance of this object 
      * @param event the event to send to the parent class of this object
      */
    public void sendEvent(JailBreakEvent event){ 
    }

    /** receives JailBreakEvents from objects
      * @param event the event received
      */
    public void receiveInternalFrameEvent(JailBreakEvent event){ 
        /* send all events to the parent class */
        this.sendEvent(event);
    }

    /** returns the item currently selected in the tree
     * @return item selected in the tree
     */    
    public Object getTreeSelection(){
        return(this.objectBrowserInternalFrame.getTreeSelection());
    }    
    
    /** returns a string representation of the object
     * @return string representation of the object
     */
    public String toString(){
        StringBuffer tostring=new StringBuffer(super.toString());
        tostring.append("\nresourcePath="+this.resourcePath);
        tostring.append("\ndefaultResourcePath="+this.defaultResourcePath);
        return(tostring.toString());
    }        
}