package com.huguesjohnson.jail.jailBreak;

/*
jailBreak - Package to launch Java Artificial Intelligence Library components
JailBreakMenu.java - Menu for main window
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.io.File;
import java.lang.Exception;
import java.lang.String;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import com.huguesjohnson.DirectoryFilter;
import com.huguesjohnson.jail.jailBreak.JailBreakUIConstants;
import com.huguesjohnson.jail.jailBreak.event.JailBreakEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakActionEvent;

/**
 * class JailBreakMenu
 * @author  Hugues Johnson
 */
public class JailBreakMenu extends JMenuBar implements ActionListener{

    /** the path to the folder containing images for current theme */
    private String resourcePath;

    /** the path to the folder containing default images */
    private String defaultResourcePath;
    
    /** Creates new JailBreakMenu
     * @param resourcePath path to the folder containing images for current theme 
     * @param defaultResourcePath path to the folder containing default images   
     */
    public JailBreakMenu(String resourcePath,String defaultResourcePath){
        super();
        this.resourcePath=resourcePath;
        this.defaultResourcePath=defaultResourcePath;
        this.createMenu();
    }

    /** creates the menu
     */
     private void createMenu(){
        this.addFileMenu();
        this.addRunMenu();
        this.addViewMenu();
        this.addHelpMenu();
     }
     
    /** adds the file menu 
     */
    private void addFileMenu(){
        JMenu menu;
        JMenu subMenu;
        JMenuItem menuItem;
        menu=new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        this.add(menu);
        /* add new menu */
        subMenu=new JMenu(JailBreakUIConstants.ACTION_NEW/*,new ImageIcon(resourcePath+"new.gif")*/);
        subMenu.setMnemonic(KeyEvent.VK_N);
        subMenu.setActionCommand(JailBreakUIConstants.ACTION_NEW);
        subMenu.addActionListener(this);
        menu.add(subMenu);
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_NEW_BPNN,new ImageIcon(resourcePath+"new_bpnn.gif"));
        menuItem.setMnemonic(KeyEvent.VK_B);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_NEW_BPNN);
        menuItem.addActionListener(this);
        subMenu.add(menuItem);
        /* add open menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_OPEN,new ImageIcon(resourcePath+"open.gif"));
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_OPEN);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add save menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_SAVE,new ImageIcon(resourcePath+"save.gif"));
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_SAVE);
        menuItem.addActionListener(this);
        //menuItem.setEnabled(false);
        menu.add(menuItem);
        /* add save as menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_SAVE_AS,new ImageIcon(resourcePath+"save_as.gif"));
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_SAVE_AS);
        //menuItem.setEnabled(false);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add a separator */
        menu.addSeparator();
        /* add exit menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_EXIT,new ImageIcon(resourcePath+"exit.gif"));
        menuItem.setMnemonic(KeyEvent.VK_X);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_EXIT);
        menuItem.addActionListener(this);
        menu.add(menuItem);
    }

    /** adds the run menu
      */
    private void addRunMenu(){
        JMenu menu;
        JMenuItem menuItem;
        menu=new JMenu("Run");
        menu.setMnemonic(KeyEvent.VK_R);
        this.add(menu);
        /* add run menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_RUN,new ImageIcon(resourcePath+"run.gif"));
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_RUN);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add run file menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_RUN_FILE,new ImageIcon(resourcePath+"run_file.gif"));
        menuItem.setMnemonic(KeyEvent.VK_F);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_RUN_FILE);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add train menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_TRAIN,new ImageIcon(resourcePath+"train.gif"));
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_TRAIN);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add train file menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_TRAIN_FILE,new ImageIcon(resourcePath+"train_file.gif"));
        menuItem.setMnemonic(KeyEvent.VK_I);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_TRAIN_FILE);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add separator */
        menu.addSeparator();
        /* add options menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_OPTIONS,new ImageIcon(resourcePath+"options.gif"));
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_OPTIONS);
        menuItem.addActionListener(this);
        /* don't have any options defined yet, leave this as a placeholder */
        menuItem.setEnabled(false);
        menu.add(menuItem);
    }
    
    /** adds the view menu
     */
    private void addViewMenu(){
        JMenu menu;
        JMenu subMenu;
        JMenuItem menuItem;
        JCheckBoxMenuItem checkMenuItem;
        menu=new JMenu("View");
        menu.setMnemonic(KeyEvent.VK_V);
        this.add(menu);
        /* add view browser menu */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_BROWSER,new ImageIcon(resourcePath+"browser.gif"),true);
        checkMenuItem.setMnemonic(KeyEvent.VK_B);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_BROWSER);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add view selected node */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_NODE,new ImageIcon(resourcePath+"node.gif"),true);
        checkMenuItem.setMnemonic(KeyEvent.VK_N);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_NODE);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add view debug messages */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_DEBUG,new ImageIcon(resourcePath+"debug_messages.gif"),false);
        checkMenuItem.setMnemonic(KeyEvent.VK_D);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_DEBUG);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add separator */
        menu.addSeparator();
        /* add view toolbar */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_TOOLBAR,new ImageIcon(resourcePath+"toolbar.gif"),true);
        checkMenuItem.setMnemonic(KeyEvent.VK_T);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_TOOLBAR);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add view menu */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_MENU,new ImageIcon(resourcePath+"menu.gif"),true);
        checkMenuItem.setMnemonic(KeyEvent.VK_M);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_MENU);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add view statusbar */
        checkMenuItem=new JCheckBoxMenuItem(JailBreakUIConstants.ACTION_VIEW_STATUS,new ImageIcon(resourcePath+"statusbar.gif"),true);
        checkMenuItem.setMnemonic(KeyEvent.VK_S);
        checkMenuItem.setActionCommand(JailBreakUIConstants.ACTION_VIEW_STATUS);
        checkMenuItem.addActionListener(this);
        menu.add(checkMenuItem);
        /* add separator */
        menu.addSeparator();
        /* add window style menu */
        subMenu=new JMenu("Window Style");
        subMenu.setMnemonic(KeyEvent.VK_W);
        menu.add(subMenu);
        LookAndFeelInfo lfi[]=UIManager.getInstalledLookAndFeels();
        int length=lfi.length;
        for(int index=0;index<length;index++){
            String name=lfi[index].getName();
            String className=lfi[index].getClassName();
            menuItem=new JMenuItem(className,new ImageIcon(resourcePath+name+".gif"));
            menuItem.setActionCommand(JailBreakUIConstants.ACTION_CHANGE_STYLE);
            menuItem.addActionListener(this);
            subMenu.add(menuItem); 
        }
        /* add themes menu */
        this.loadThemes(menu);
    }
    
     /** loadThemes opens up the working directory, loads available themes, and adds them under parent menu
     * @param parentMenu the menu to add the themes to
     */
     private void loadThemes(JMenu parentMenu){
          String imageName;
          JMenuItem menuItem;
          JMenu themeMenu=new JMenu("Theme");
          themeMenu.setMnemonic(KeyEvent.VK_H);
          parentMenu.add(themeMenu);
          String themeList[]={""};
          try{
               /* under some IDEs (such as Forte and Visual Cafe) the next line will not work because System.getProperty("user.dir") returns a path to the IDE and not to the working folder */
               String themePath=new String(System.getProperty("user.dir")+File.separator+"themes"+File.separator);
               File f=new File(themePath);
               themeList=new String[f.list().length];
               themeList=f.list(new DirectoryFilter());
               for(int q=0;q<themeList.length;q++){
                    imageName=themePath+themeList[q]+File.separator+"theme.gif";
                    /* replace underscores with blanks when adding to menu */
                    menuItem=new JMenuItem(themeList[q].replace('_',' '),new ImageIcon(imageName));
                    menuItem.setActionCommand(JailBreakUIConstants.ACTION_CHANGE_THEME+"|"+themeList[q]);
                    menuItem.addActionListener(this);
                    themeMenu.add(menuItem);
               }
          } catch(Exception x){
               x.printStackTrace();
          }
     }    
    
    /** adds the help menu 
     */
    private void addHelpMenu(){
        JMenu menu;
        JMenuItem menuItem;
        menu=new JMenu("Help");
        menu.setMnemonic(KeyEvent.VK_H);
        this.add(menu);
        /* add index menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_HELP_INDEX,new ImageIcon(resourcePath+"readme.gif"));
        menuItem.setMnemonic(KeyEvent.VK_I);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_HELP_INDEX);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add readme menu */
        //menuItem=new JMenuItem(JailBreakUIConstants.ACTION_HELP_README,new ImageIcon(resourcePath+"readme.gif"));
        //menuItem.setMnemonic(KeyEvent.VK_R);
        //menuItem.setActionCommand(JailBreakUIConstants.ACTION_HELP_README);
        //menuItem.addActionListener(this);
        //menu.add(menuItem);
        /* add FAQ menu */
        //menuItem=new JMenuItem(JailBreakUIConstants.ACTION_HELP_FAQ,new ImageIcon(resourcePath+"faq.gif"));
        //menuItem.setMnemonic(KeyEvent.VK_F);
        //menuItem.setActionCommand(JailBreakUIConstants.ACTION_HELP_FAQ);
        //menuItem.addActionListener(this);
        //menu.add(menuItem);
        /* add API menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_HELP_API,new ImageIcon(resourcePath+"api.gif"));
        menuItem.setMnemonic(KeyEvent.VK_D);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_HELP_API);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        /* add sperarator */
        menu.addSeparator();
        /* add about menu */
        menuItem=new JMenuItem(JailBreakUIConstants.ACTION_HELP_ABOUT,new ImageIcon(resourcePath+"about.gif"));
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.setActionCommand(JailBreakUIConstants.ACTION_HELP_ABOUT);
        menuItem.addActionListener(this);
        menu.add(menuItem);
    }
     
    /** actionPerformed listener for all events
     * @param actionEvent the event received
     */
    public void actionPerformed(ActionEvent actionEvent){
        /* send action event to the parent class */
        String actionCommand=actionEvent.getActionCommand();
        String parameter=null;
        if(actionCommand.equals(JailBreakUIConstants.ACTION_CHANGE_STYLE)||actionCommand.equals(JailBreakUIConstants.ACTION_CHANGE_THEME)){
            parameter=((JMenuItem)actionEvent.getSource()).getText();
        } else if(actionEvent.getSource() instanceof JCheckBoxMenuItem){
            parameter=String.valueOf(((JCheckBoxMenuItem)actionEvent.getSource()).isSelected());
        }
        this.sendEvent(new JailBreakActionEvent(actionCommand,parameter));
    }

    /** sendEvent override when creating a new instance of this object 
      * @param event the event to send to the parent class of this object
      */
    public void sendEvent(JailBreakEvent event){ 
    }
}