package com.huguesjohnson.jail.jailBreak;

/*
jailBreak - Package to launch Java Artificial Intelligence Library components
JailBreakNetworkTree.java - JTree which displays a neural network or decision tree
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.lang.Exception;
import java.lang.String;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JTree;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import com.huguesjohnson.jail.jailBreak.JailBreakUIConstants;
import com.huguesjohnson.jail.jailBreak.event.JailBreakEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakTreeSelectionEvent;
import com.huguesjohnson.jail.jailBreak.tree.NeuralNetworkTreeNode;
import com.huguesjohnson.jail.neuralNetwork.BackPropagationNetwork;
import com.huguesjohnson.jail.neuralNetwork.Layer;
import com.huguesjohnson.jail.neuralNetwork.LayerException;
import com.huguesjohnson.jail.neuralNetwork.Neuron;

/** JailBreakNetworkTree displays a neural network or decision tree
 * @author Hugues Johnson
 */
public class JailBreakNetworkTree extends JTree implements TreeModelListener,TreeSelectionListener{

    /** the path to the folder containing images for current theme */
    private String resourcePath;

    /** the path to the folder containing default images */
    private String defaultResourcePath;
    
    /** Creates new JailBreakNetworkTree
     * @param resourcePath path to the folder containing images for current theme 
     * @param defaultResourcePath path to the folder containing default images   
     */
    public JailBreakNetworkTree(String resourcePath,String defaultResourcePath){
        super(new NeuralNetworkTreeNode(new String(),"<null>"));
        this.resourcePath=resourcePath;
        this.defaultResourcePath=defaultResourcePath;
        this.addTreeSelectionListener(this);
    }
    
    /** updates the tree based on the object passed
     * @param bpNetwork a backpropagation neural network to display
     */
    public void updateTree(BackPropagationNetwork bpNetwork){
        /* is this the same item that's already selected? */
        boolean newModel=true;
        if(this.getLastSelectedPathComponent()!=null){
            Object selectedUserObject=((DefaultMutableTreeNode)this.getLastSelectedPathComponent()).getUserObject();
            if(selectedUserObject!=null){
                /* should .equals be overridden and used instead? */
                if(selectedUserObject==bpNetwork){
                    newModel=false;
                }
            }
        }
        if(newModel){
            /* create root node */
            NeuralNetworkTreeNode rootNode=new NeuralNetworkTreeNode(bpNetwork,"Network");
            DefaultTreeModel treeModel=new DefaultTreeModel(rootNode);
            try{
                /* add input layer */
                Layer layer=(Layer)bpNetwork.getInputLayer();
                NeuralNetworkTreeNode inputLayerNode=new NeuralNetworkTreeNode(layer,"Input Layer");
                treeModel.insertNodeInto(inputLayerNode,rootNode,0);
                int size=layer.getSize();
                for(int index=0;index<size;index++){
                    Neuron neuron=layer.getNeuron(index);
                    NeuralNetworkTreeNode neuronNode=new NeuralNetworkTreeNode(neuron,"Neuron["+index+"]");
                    treeModel.insertNodeInto(neuronNode,inputLayerNode,index);
                }
                /* add hidden layer */
                layer=(Layer)bpNetwork.getHiddenLayer();
                NeuralNetworkTreeNode hiddenLayerNode=new NeuralNetworkTreeNode(layer,"Hidden Layer");
                treeModel.insertNodeInto(hiddenLayerNode,rootNode,1);
                size=layer.getSize();
                for(int index=0;index<size;index++){
                    Neuron neuron=layer.getNeuron(index);
                    NeuralNetworkTreeNode neuronNode=new NeuralNetworkTreeNode(neuron,"Neuron["+index+"]");
                    treeModel.insertNodeInto(neuronNode,hiddenLayerNode,index);
                }            
                /* add output layer */
                layer=(Layer)bpNetwork.getOutputLayer();
                NeuralNetworkTreeNode outputLayerNode=new NeuralNetworkTreeNode(layer,"Output Layer");
                treeModel.insertNodeInto(outputLayerNode,rootNode,2);
                size=layer.getSize();
                for(int index=0;index<size;index++){
                    Neuron neuron=layer.getNeuron(index);
                    NeuralNetworkTreeNode neuronNode=new NeuralNetworkTreeNode(neuron,"Neuron["+index+"]");
                    treeModel.insertNodeInto(neuronNode,outputLayerNode,index);
                }
            } catch(LayerException lx){
                lx.printStackTrace();
            }
            /* set the model for the tree */
            this.setModel(treeModel);
        }
    }
    
    /** sendEvent override when creating a new instance of this object 
      * @param event the event to send to the parent class of this object
      */
    public void sendEvent(JailBreakEvent event){ 
    }    

    /** receiveEvent receives JailBreakEvents from objects
      * @param event the event received
      */
    public void receiveEvent(JailBreakEvent event){ 
        /* send all events to the parent class */
        this.sendEvent(event);
    }  
    
    /** returns the underlying object of the selected node
     * @return the underlying object of the selected node
     */    
    public Object getSelectedObject(){
        Object selectedObject=this.getLastSelectedPathComponent();
        if(selectedObject==null){
            selectedObject=this.getModel().getRoot();
        } 
        if(selectedObject instanceof DefaultMutableTreeNode){
            return(((DefaultMutableTreeNode)selectedObject).getUserObject());
        } else{
            return(selectedObject);
        }
    }
    
    /** event that indicates the tree structure has changed
     * @param treeModelEvent the event
     */
    public void treeStructureChanged(TreeModelEvent treeModelEvent){
    }
    
    /** event that indicates nodes have been inserted
     * @param treeModelEvent the event
     */
    public void treeNodesInserted(TreeModelEvent treeModelEvent){
    }
    
    /** event that indicates nodes have been removed
     * @param treeModelEvent the event
     */
    public void treeNodesRemoved(TreeModelEvent treeModelEvent){
    }
    
    /** event that indicates nodes have been changed
     * @param treeModelEvent the event
     */
    public void treeNodesChanged(TreeModelEvent treeModelEvent){
    }
    
    /** event that indicates a node's text has changed
     * @param treeSelectionEvent the event
     */
    public void valueChanged(TreeSelectionEvent treeSelectionEvent){
        this.sendEvent(new JailBreakTreeSelectionEvent(((DefaultMutableTreeNode)this.getLastSelectedPathComponent()).getUserObject()));
    }
}