package com.huguesjohnson.jail.jailBreak.wizard;

/*
jailBreak - Package to launch Java Artificial Intelligence Library components
JailBreakStartupWizard.java - startup dialog 
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.lang.String;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import com.huguesjohnson.jail.jailBreak.JailBreakUIConstants;
import com.huguesjohnson.jail.jailBreak.event.JailBreakActionEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakEvent;
import com.huguesjohnson.jail.jailBreak.event.JailBreakOpenFileEvent;
import com.huguesjohnson.jail.jailBreak.fileFilter.BackPropagationNetworkFilter;

/** JailBreakStartupWizard startup dialog
 * @author Hugues Johnson
 */
public class JailBreakStartupWizard extends JDialog implements ListSelectionListener,ActionListener{
    /** the path to the folder containing images for current theme */
    private String resourcePath;

    /** the working directory */
    private String workingDirectory;

    /** the action selected by the user when the dialog is exited */
    private JailBreakEvent exitAction;
    
    /** list of new components */
     private JList newList;

    /** underlying model for list of new components */
     private DefaultListModel newListModel;
     
    /** list of recent files */
     //private JList recentList;

    /** underlying model for list of recent files */
    //private DefaultListModel recentListModel;
     
     /** textfield to allow user to input a file */
     private JTextField fileTextField;
     
    /** constructor
     * @param parent the parent frame to this wizard
     * @param resourcePath the path to directory containing images for this dialog
     * @param workingDirectory the working directory for this dialog
     */
    public JailBreakStartupWizard(Frame parent,String resourcePath,String workingDirectory){
        super(parent,true);
        this.setTitle(JailBreakUIConstants.CAPTION_WINDOW);
        this.resourcePath=resourcePath;
        this.workingDirectory=workingDirectory;
        this.exitAction=new JailBreakActionEvent(JailBreakUIConstants.ACTION_EXIT);
        this.addComponents();
        /* size the window */
        this.setSize(480,320);
        /* center the window */
        Dimension screenDimension=Toolkit.getDefaultToolkit().getScreenSize();
        Rectangle windowDimension=this.getBounds();
        this.setLocation((screenDimension.width-windowDimension.width)/2,(screenDimension.height-windowDimension.height)/2);        
        /* set the focus to the first item in newList */
        this.newList.setSelectedIndex(0);
        this.newList.grabFocus();
    }
    
    /** returns the last action selected by the user
     * @return last action selected by user
     */
    public JailBreakEvent getExitActionEvent(){
        return(this.exitAction);
    }
    
    /** adds components to the dialog
     */
    private void addComponents(){
        /* set layout for this dialog */
        this.getContentPane().setLayout(new BoxLayout(this.getContentPane(),BoxLayout.Y_AXIS));
        /* add panel for new components */
        JPanel panel=new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"New"));
        /* initialize new list */
        this.newListModel=new DefaultListModel();
        this.newListModel.addElement(new String(JailBreakUIConstants.ACTION_NEW_BPNN));
        this.newList=new JList(this.newListModel);
        this.newList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.newList.addListSelectionListener(this);
        JScrollPane scrollPane=new JScrollPane(newList);
        panel.add(scrollPane);
        this.getContentPane().add(panel);
        /* add panel for recent components */
        panel=new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Recent"));
        /* initialize new list */
        //this.recentListModel=new DefaultListModel();
        //TODO add recent files here
        //this.recentListModel.addElement(new String(JailBreakUIConstants.ACTION_NEW_BPNN));
        //this.recentList=new JList(this.recentListModel);
        //this.recentList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        //this.recentList.addListSelectionListener(this);
        //scrollPane=new JScrollPane(recentList);
        //panel.add(scrollPane);
        //this.getContentPane().add(panel);    
        /* add panel for textbox with browse button */
        panel=new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Open"));
        panel.setLayout(new BoxLayout(panel,BoxLayout.X_AXIS));
        this.fileTextField=new JTextField();
        scrollPane=new JScrollPane(this.fileTextField);
        panel.add(scrollPane);
        JButton button=new JButton(JailBreakUIConstants.ACTION_OPEN);
        button.setMnemonic(KeyEvent.VK_O);
        button.setActionCommand(JailBreakUIConstants.ACTION_OPEN);
        button.addActionListener(this);
        panel.add(button);
        this.getContentPane().add(panel);
        /* add panel for ok and cancel buttons */
        panel=new JPanel(new BorderLayout());
        panel.setLayout(new BoxLayout(panel,BoxLayout.X_AXIS));
        panel.setAlignmentX(0.0f);
        button=new JButton(JailBreakUIConstants.ACTION_OK);
        button.setActionCommand(JailBreakUIConstants.ACTION_OK);
        button.addActionListener(this);
        panel.add(button);
        button=new JButton(JailBreakUIConstants.ACTION_CANCEL);
        button.setActionCommand(JailBreakUIConstants.ACTION_CANCEL);
        button.addActionListener(this);
        panel.add(button);
        this.getContentPane().add(panel);
    }
    
    /** occurs when a value is selected from a list 
     * @param listSelectionEvent the selection event
     */
    public void valueChanged(ListSelectionEvent listSelectionEvent){
        JList source=(JList)listSelectionEvent.getSource();
        if(source==this.newList){
            String selectedValue=(String)this.newList.getSelectedValue();
            if(selectedValue.equals(JailBreakUIConstants.ACTION_NEW_BPNN)){
                this.exitAction=new JailBreakActionEvent(JailBreakUIConstants.ACTION_NEW_BPNN);
            }
        //} else if(source==this.recentList){
        }
    }
    
     /** occurs when a button is clicked
     * @param actionEvent the action received
     */
     public void actionPerformed(ActionEvent actionEvent){
        String actionCommand=actionEvent.getActionCommand();
        if(actionCommand==JailBreakUIConstants.ACTION_OPEN){
            /* show file chooser */
            JFileChooser fc=new JFileChooser();
            /* add BackPropagationNetwork filter  */
            fc.addChoosableFileFilter(new BackPropagationNetworkFilter());
            fc.setAcceptAllFileFilterUsed(false);
            int returnValue=fc.showOpenDialog(this);
            if(returnValue==fc.APPROVE_OPTION){
                String selectedFile=fc.getCurrentDirectory().getPath()+File.separator+fc.getSelectedFile().getName();
                this.exitAction=new JailBreakOpenFileEvent(selectedFile,fc.getFileFilter());
                this.fileTextField.setText(selectedFile);
            }
        } else if(actionCommand==JailBreakUIConstants.ACTION_OK){
            this.dispose();
        } else if(actionCommand==JailBreakUIConstants.ACTION_CANCEL){
            this.exitAction=new JailBreakActionEvent(JailBreakUIConstants.ACTION_EXIT);
            this.dispose();
        }
     }    
    
}