package com.huguesjohnson.jail.jdt;

/*
Jdt (Java Decision Tree) - Package to create and display a decision tree
JdtUIFilePanel.java - swing panel to display file options
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import com.huguesjohnson.jail.jdt.JdtUIFilePanelBox;

/** JdtUIFilePanel
 * @author Hugues Johnson
 */
public class JdtUIFilePanel extends JPanel implements ActionListener{
    /** index of names file field */    
    public static final int FIELD_NAMES=0;

    /** index of training file field */    
    public static final int FIELD_TRAINING=1;

    /** index of test file field */    
    public static final int FIELD_TEST=2;

    /** index of log file field */    
    public static final int FIELD_LOG=3;
    
    /** index of report file field */        
    public static final int FIELD_REPORT=4;
    
    /** reference to the names file box */
    private JdtUIFilePanelBox namesBox;

    /** reference to the training file box */
    private JdtUIFilePanelBox trainingBox;
    
    /** reference to the test file box */
    private JdtUIFilePanelBox testBox;
    
    /** reference to the log file box */
    private JdtUIFilePanelBox logBox;
    
    /** reference to the report file box */
    private JdtUIFilePanelBox reportBox;
    
    /** constructor
     */
    public JdtUIFilePanel(){
        this.addComponents();
    }

    /** adds components to the panel
     */
    private void addComponents(){
        this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
        this.add(new JLabel("File List"));
        this.namesBox=new JdtUIFilePanelBox("Names Files");
        this.add(namesBox);        
        this.trainingBox=new JdtUIFilePanelBox("Training Data");
        this.add(trainingBox);
        this.testBox=new JdtUIFilePanelBox("Test Data");
        this.add(testBox);
        this.reportBox=new JdtUIFilePanelBox("Report");
        this.add(reportBox);
        this.logBox=new JdtUIFilePanelBox("Debug Log");
        this.add(logBox);
    }
    
    /** returns the value of a field displayed in the panel
     * @param field index of the field to return
     * @return the value of the field
     */
    public String getField(int field){
        switch(field){
            case FIELD_NAMES: return(this.namesBox.getFilePath());
            case FIELD_TRAINING: return(this.trainingBox.getFilePath());
            case FIELD_TEST: return(this.testBox.getFilePath());
            case FIELD_LOG: return(this.logBox.getFilePath());
            case FIELD_REPORT: return(this.reportBox.getFilePath());
            default: return(new String());
        }
    }

    /** set the value of a field displayed in the panel
     * @param field index of the field 
     * @param text the new value of the field
     */
    public void setField(int field,String text){
        switch(field){
            case FIELD_NAMES: this.namesBox.setFilePath(text); break;
            case FIELD_TRAINING: this.trainingBox.setFilePath(text); break;
            case FIELD_TEST: this.testBox.setFilePath(text); break;
            case FIELD_LOG: this.logBox.setFilePath(text); break;
            case FIELD_REPORT: this.reportBox.setFilePath(text); break;
        }
    }    
    
    /** receives all swing actions for this panel
     * @param e the event
     */
    public void actionPerformed(ActionEvent e){
    }
    
    /** returns a string representation of the object
     * @return string representation of the object
     */
    public String toString(){
        String tostring=new String(super.toString());
        tostring+="\nnamesBox="+this.namesBox.toString();
        tostring+="\ntrainingBox="+this.trainingBox.toString();
        tostring+="\ntestBox="+this.testBox.toString();
        tostring+="\nlogBox="+this.logBox.toString();
        return(tostring);
    }    
}