package com.huguesjohnson.jail.jdt;

/*
Jdt (Java Decision Tree) - Package to create and display a decision tree
JdtUIMenu.java - menu for user interface
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.io.File;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.ImageIcon;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

/** Class JdtUIMenu creates a JMenuBar that provides access to valid Jdt commands
* @author Hugues Johnson
*/
public class JdtUIMenu extends JMenuBar implements ActionListener{
    /** the directory that contains images for this menu */ 
    private String resourcePath;

     /** Default constructor creates the toolbar, draws images, and binds actionCommands.
     */
     public JdtUIMenu(){
          /* set the resource path
          under some IDEs (such as Forte and Visual Cafe) the next line will not work because System.getProperty("user.dir") returns a path to the IDE and not to the working folder */
          String defaultResourcePath=new String(System.getProperty("user.dir")+File.separator+"themes"+File.separator+"Default"+File.separator);
          this.setResourcePath(defaultResourcePath);          
          this.createMenu();
     }
     
     /** Constructor creates the toolbar, draws images, binds actionCommands, and sets the resource path.
     * @param resourcePath path to folder containing images for menu
     */
     public JdtUIMenu(String resourcePath){
          this.setResourcePath(resourcePath);
          this.createMenu();
     }
     
     /** creates the menu
     */
     private void createMenu(){
          this.addFileMenu();
          this.addViewMenu();
          this.addHelpMenu();
     }
     
     /** adds the file menu
      */
     private void addFileMenu(){
          JMenu menu;
          JMenuItem menuItem;
          
          menu=new JMenu("File");
          menu.setMnemonic(KeyEvent.VK_F);
          this.add(menu);
          menuItem=new JMenuItem("Exit"/*,new ImageIcon(resourcePath+"exit.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_X);
          menuItem.setActionCommand(JdtUIConstants.ACTION_EXIT);
          menuItem.addActionListener(this);
          menu.add(menuItem);
     }
     
     /** adds the help menu
      */     
     private void addHelpMenu(){
          JMenu menu;
          JMenuItem menuItem;
          menu=new JMenu("Help");
          menu.setMnemonic(KeyEvent.VK_H);
          this.add(menu);
          menuItem=new JMenuItem("About.."/*,new ImageIcon(resourcePath+"about.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_A);
          menuItem.setActionCommand(JdtUIConstants.ACTION_ABOUT);
          menuItem.addActionListener(this);
          menu.add(menuItem);
     }

     /** adds the view menu
      */
     private void addViewMenu(){
          JMenu menu;
          JMenuItem menuItem;
          menu=new JMenu("View");
          menu.setMnemonic(KeyEvent.VK_V);
          this.add(menu);
          menuItem=new JMenuItem("Welcome"/*,new ImageIcon(resourcePath+"about.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_W);
          menuItem.setActionCommand(JdtUIConstants.ACTION_SET_PANE_WELCOME);
          menuItem.addActionListener(this);
          menu.add(menuItem);
          menuItem=new JMenuItem("Files"/*,new ImageIcon(resourcePath+"about.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_F);
          menuItem.setActionCommand(JdtUIConstants.ACTION_SET_PANE_FILE);
          menuItem.addActionListener(this);
          menu.add(menuItem);
          menuItem=new JMenuItem("Run"/*,new ImageIcon(resourcePath+"about.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_R);
          menuItem.setActionCommand(JdtUIConstants.ACTION_SET_PANE_RUN);
          menuItem.addActionListener(this);
          menu.add(menuItem);
          menuItem=new JMenuItem("Tree"/*,new ImageIcon(resourcePath+"about.gif")*/);
          menuItem.setMnemonic(KeyEvent.VK_T);
          menuItem.setActionCommand(JdtUIConstants.ACTION_SET_PANE_TREE);
          menuItem.addActionListener(this);
          menu.add(menuItem);
     }     
     
     /** sets the path to the resource (image) directory
     * @param resourcePath a path to the resource (image) directory 
     */
     public void setResourcePath(String resourcePath){
          /* test if resource path is a valid directory */
          File temp=new File(resourcePath);
          if(temp.isDirectory()){
               this.resourcePath=resourcePath;
          }
     }     

     /** returns a String representation of this object
      * @return String representation of this object
      */
     public String toString(){
          String tostring=new String(this.getClass().getName());
          return(tostring);
     }     
     
     /** actionPerformed empty method meant to be overidden by parent window.
     * @param e ActionEvent received
     */
     public void actionPerformed(final ActionEvent e){
     }
}