package com.huguesjohnson.jail.jdt;

/*
Jdt (Java Decision Tree) - Package to create and display a decision tree
JdtUIWindow.java - main window for program 
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.String;
import java.lang.System;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSplitPane;
import com.huguesjohnson.jail.jdt.JdtUIFilePanel;
import com.huguesjohnson.jail.jdt.JdtUIMenu;
import com.huguesjohnson.jail.jdt.JdtUIRunPanel;
import com.huguesjohnson.jail.jdt.JdtUITreePanel;
import com.huguesjohnson.jail.jdt.JdtUIWelcomePanel;

/** class JdtUIWindow
 * @author Hugues Johnson
 */
public class JdtUIWindow extends JFrame{
    /** directory containing images for the frame */
    private String resourcePath;
    
    /** reference to the progress bar */
    private JProgressBar progress;
    
    /** reference to the split pane */
    private JSplitPane splitPane;
    
    /** reference to the navigate panel */
    private JdtUINavigatePanel leftPane;
    
    /** reference to the welcome panel */
    private JdtUIWelcomePanel welcomePane;
    
    /** reference to the file panel */
    private JdtUIFilePanel filePane;
    
    /** reference to the run panel */
    private JdtUIRunPanel runPane;
    
    /** reference to the tree panel */
    private JdtUITreePanel treePane;
    
    /** reference to the menu */
    private JdtUIMenu menu;
    
    /** index of the welcome panel */
    public final static int PANE_WELCOME=0;
    
    /** index of the file panel */
    public final static int PANE_FILE=1;
    
    /** index of the run panel */
    public final static int PANE_RUN=2;
    
    /** index of the tree panel */
    public final static int PANE_TREE=3;
    
    /** constructor
     * @param caption caption for the window
     */
    public JdtUIWindow(String caption){
        /* invoke constructor for JFrame */
        super(caption);
        /* set the resource path
         under some IDEs (such as Forte and Visual Cafe) the next line will not work because System.getProperty("user.dir") returns a path to the IDE and not to the working folder */
        this.resourcePath=new String(System.getProperty("user.dir")+File.separator+"themes"+File.separator+"Jdt"+File.separator);
        /* add components */
        this.addComponents();
    }
    
    /** adds components to the frame
     */
    private void addComponents(){
        /* create content pane */
        Container contentPane=this.getContentPane();
        /* initialize panels & override actionPerformed event */
        this.leftPane=new JdtUINavigatePanel(this.resourcePath){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        this.welcomePane=new JdtUIWelcomePanel(){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        this.filePane=new JdtUIFilePanel(){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        this.runPane=new JdtUIRunPanel(){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        this.treePane=new JdtUITreePanel(){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        /* add menu */
        this.menu=new JdtUIMenu(this.resourcePath){ public void actionPerformed(final ActionEvent e){ actionListener(e);}};
        this.setJMenuBar(this.menu);
        /* add progress bar */
        this.progress=new JProgressBar();
        this.progress.setStringPainted(true);
        contentPane.add(this.progress,BorderLayout.SOUTH);
        /* create split pane to hold components */
        //JPanel panel=new JPanel();
        //panel.setMinimumSize(new Dimension(400,400));
        //this.splitPane=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,this.leftPane,panel);
        this.splitPane=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,this.leftPane,this.welcomePane);
        this.splitPane.setOneTouchExpandable(true);
        this.setRightPane(this.PANE_WELCOME);
        /* size components */
        /** @TODO store size settings externally */
        this.splitPane.setDividerLocation(64+this.splitPane.getDividerSize());
        this.leftPane.setMinimumSize(new Dimension(64+this.splitPane.getDividerSize(),400));
        /* add split pane */
        contentPane.add(this.splitPane);
        /* add listener for close event */
        addWindowListener(new WindowAdapter(){ public void windowClosing(WindowEvent e){closeWindow();}});
    }

    /** sets the contents of the right pane
     * @param pane panel to display in the right pane
     */
    public void setRightPane(int pane){
        int dividerLocation=this.splitPane.getDividerLocation();
        switch(pane){
            case PANE_WELCOME: 
                this.splitPane.setRightComponent(this.welcomePane); 
                this.setProgress(JdtUIConstants.STATUS_WELCOME,0);
                break;
            case PANE_FILE: 
                this.splitPane.setRightComponent(this.filePane); 
                this.setProgress(JdtUIConstants.STATUS_SELECT_FILES,0);
                break;
            case PANE_RUN: 
                boolean namesFile=(this.filePane.getField(this.filePane.FIELD_NAMES).trim().length()>0);
                boolean trainFile=(this.filePane.getField(this.filePane.FIELD_TRAINING).trim().length()>0);
                boolean testFile=(this.filePane.getField(this.filePane.FIELD_TEST).trim().length()>0);
                boolean logFile=(this.filePane.getField(this.filePane.FIELD_LOG).trim().length()>0);
                boolean reportFile=(this.filePane.getField(this.filePane.FIELD_REPORT).trim().length()>0);
                if((namesFile==false)||(trainFile==false)||(testFile==false)){
                    this.runPane.setTestTrainDataCheckbox(false,false);
                    this.runPane.setTestTestDataCheckbox(false,false);
                    this.runPane.setCreateExecutionLogCheckbox(false,false);
                    this.runPane.setCreateReportCheckbox(false,false);
                    this.runPane.setRunButtonEnabled(false);
                    this.setProgress(JdtUIConstants.STATUS_SELECT_FILES,0);
                } else{
                    this.runPane.setTestTrainDataCheckbox(this.runPane.getTestTrainDataCheckboxValue(),true);
                    this.runPane.setTestTestDataCheckbox(this.runPane.getTestTestDataCheckboxValue(),true);
                    this.runPane.setCreateExecutionLogCheckbox(this.runPane.getCreateExecutionCheckboxValue(),true);
                    this.runPane.setCreateReportCheckbox(this.runPane.getCreateReportCheckboxValue(),true);
                    this.runPane.setRunButtonEnabled(true);                
                    this.setProgress(JdtUIConstants.STATUS_RUN,0);
                }
                if(logFile==false){
                    this.runPane.setCreateExecutionLogCheckbox(false,false);
                } 
                if(reportFile==false){
                    this.runPane.setCreateReportCheckbox(false,false);
                } 
                this.splitPane.setRightComponent(this.runPane); 
                break;
            case PANE_TREE: 
                this.splitPane.setRightComponent(this.treePane); 
                break;
        }
        this.splitPane.setDividerLocation(dividerLocation);
    }
    
    /** returns the path to the resource directory
     * @return path to the resource directory
     */
     public String getResourcePath(){
         return(this.resourcePath);
     }
     
    /** override this routine when creating a new window
     * @param e ActionEvent received by UI
     */
     public void actionListener(final ActionEvent e){
          /* override this */
     }

     /** returns the name of the names file
      * @return file name
      */
     public String getNamesFileName(){
         if(this.filePane!=null){
             return(this.filePane.getField(filePane.FIELD_NAMES));
         } else{
            return(new String());
         }
     }
     
     /** returns the name of the training file
      * @return file name
      */
     public String getTrainingFileName(){
         if(this.filePane!=null){
             return(this.filePane.getField(filePane.FIELD_TRAINING));
         } else{
            return(new String());
         }
     }     
     
    /** returns the name of the test file 
      * @return file name
      */
    public String getTestFileName(){
         if(this.filePane!=null){
             return(this.filePane.getField(filePane.FIELD_TEST));
         } else{
            return(new String());
         }
     }
     
     /** returns the name of the report
      * @return file name
      */
     public String getReportFileName(){
         if(this.filePane!=null){
             return(this.filePane.getField(filePane.FIELD_REPORT));
         } else{
            return(null);
         }
     }     

     /** returns the name of the log file 
      * @return file name
      */
     public String getDebugFileName(){
         if(this.filePane!=null){
             return(this.filePane.getField(filePane.FIELD_LOG));
         } else{
            return(null);
         }
     }    

     /** sets the name and path of the names file
      * @param fileName new file name
      */
     public void setNamesFileName(String fileName){
         if(this.filePane!=null){
             this.filePane.setField(filePane.FIELD_NAMES,fileName);
         }
     }     
     
     /** sets the name and path of the training file
      * @param fileName new file name
      */
     public void setTrainingFileName(String fileName){
         if(this.filePane!=null){
             this.filePane.setField(filePane.FIELD_TRAINING,fileName);
         }
     }     
     
     /** sets the name and path of the test file
      * @param fileName new file name
      */
    public void setTestFileName(String fileName){
         if(this.filePane!=null){
             this.filePane.setField(filePane.FIELD_TEST,fileName);
         }
     }
     
     /** sets the name and path of the report file
      * @param fileName new file name
      */
     public void setReportFileName(String fileName){
         if(this.filePane!=null){
             this.filePane.setField(filePane.FIELD_REPORT,fileName);
         }
     }     

     /** sets the name and path of the log file
      * @param fileName new file name
      */
     public void setDebugFileName(String fileName){
         if(this.filePane!=null){
             this.filePane.setField(filePane.FIELD_LOG,fileName);
         }
     }         
     
     
     /** sets the percentage and text of the progress bar
      * @param message the caption of the progress bar
      * @param percent the percent complete of the progress bar
      */
     public void setProgress(String message,int percent){
         if(this.progress!=null){
            this.progress.setString(message);
            this.progress.setValue(percent);
         }
     }
     
     /** override this routine when creating a new window
     */
     public void closeWindow(){
          /* override this */
     }
     
    /** returns a string representation of the object
     * @return string representation of the object
     */
    public String toString(){
        String tostring=new String(super.toString());
        tostring+="\nresourcePath="+this.resourcePath;
        tostring+="\nprogress="+this.progress.toString();
        tostring+="\nsplitPane="+this.splitPane.toString();
        tostring+="\nleftPane="+this.leftPane.toString();
        tostring+="\nwelcomePane="+this.welcomePane.toString();
        tostring+="\nfilePane="+this.filePane.toString();
        tostring+="\nrunPane="+this.runPane.toString();
        tostring+="\ntreePane="+this.treePane.toString();
        return(tostring);
    }     
}