package com.huguesjohnson.jail.neuralNetwork;

/*
neuralNetwork - Package of neural network objects
NeuralNetwork.java - Abstract base class for all neural networks
Copyright  (C) 2000-2003 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import java.io.Serializable;
import java.lang.Object;
import java.lang.Runnable;
import java.lang.String;
import java.lang.StringBuffer;
import java.lang.System;
import java.lang.Thread;
import com.huguesjohnson.LogFile;
import com.huguesjohnson.jail.neuralNetwork.NeuralNetworkException;

/** class NeuralNetwork, base class for neural network objects
 * contains a functionality that is common to all neural networks
 */
public abstract class NeuralNetwork extends Object implements Runnable,Serializable{
    /** log file for network */
    //private LogFile logFile;
    
    /** thread for this class */
    private Thread myThread;
    
    /* constructor(s) */
    
    /** default constructor
     */
    public NeuralNetwork(){
        myThread=null;
        //this.logFile=new LogFile(this.getClass().getName()+"_"+System.currentTimeMillis()+".log");
    }
    
    /** constructor
     * @param logFile path to log file
     */
    public NeuralNetwork(String logFile){
        myThread=null;
        //this.logFile=new LogFile(logFile);
    }
    
    /** destructor
     */
    public void finalize(){
        //if(this.logFile!=null){
        //    this.logFile.close();
        //}
    }
    
    /* methods common to all NeuralNetworks */
    
    /** append a line to the log file
     * @param line the line to append
     */
    protected final void log(String line){
        //if(this.logFile!=null){
        //    this.logFile.log(line);
        //}
    }

    /** begins thread execution
     */
    public void start(){
        if(myThread==null){
            myThread=new Thread(this);
        }
        myThread.start();
    }

    /** stops thread execution
     */
    public void stop(){
        myThread=null;    
    }    
    
    /* public methods that can be overridden by subclasses */    

    /** run the neural network
     */
    public void run(){
    }    

    /** returns a string representation of the object
     * @return string representation of the object
     */
    public String toString(){
        StringBuffer tostring=new StringBuffer();
        tostring.append(this.getClass().getName());
        return(tostring.toString());
    }

    /* the remaining methods must be overridden by subclasses */
    
    /** train the network
     * @param iterations the number of training iterations to run
     */
    public abstract void train(int iterations);
    
    /** returns the error for the network
     * @return error for the network
     */
    public abstract double getError();
    
    /** returns the output for the network
     * @return output for the network
     */
    public abstract double[] getOutput();
    
    /** sets the input to the network
     * @param input the input to the network
     */
    public abstract void setInput(double[] input);

    /** sets the desired output of the network
     * @param desiredOutput the desired output of the network
     */
    public abstract void setDesiredOutput(double[] desiredOutput);
}