package com.huguesjohnson.jail.neuralNetwork;

import java.lang.IndexOutOfBoundsException;
import java.lang.Math;
import java.lang.Object;
import java.lang.String;
import java.lang.StringBuffer;
import java.lang.System;
import com.huguesjohnson.InputParser;
import com.huguesjohnson.jail.neuralNetwork.BackPropagationLayer;
import com.huguesjohnson.jail.neuralNetwork.KohonenLayer;
import com.huguesjohnson.jail.neuralNetwork.KohonenNeuron;
import com.huguesjohnson.jail.neuralNetwork.Layer;
import com.huguesjohnson.jail.neuralNetwork.LayerException;
import com.huguesjohnson.jail.neuralNetwork.NetworkVirusLamstarNetwork;
import com.huguesjohnson.jail.neuralNetwork.NeuralNetwork;
import com.huguesjohnson.jail.neuralNetwork.NeuralNetworkException;
import com.huguesjohnson.jail.neuralNetwork.Neuron;
import com.huguesjohnson.jail.neuralNetwork.NeuronException;
import com.huguesjohnson.jail.neuralNetwork.Perceptron;

/** class TestRoutines a collection of test routines for neural network objects
  */
public class TestRoutines extends Object{
    
    /** test basic functionalilty of perceptron by training it to recognize odd numbers
     */
    public static void testPerceptron(){
        System.out.println("\nEnter testPerceptron()");
        double[] binaryData={-1.0D,-1.0D,-1.0D,-1.0D, //0
                                        -1.0D,-1.0D,-1.0D,1.0D, //1
                                        -1.0D,-1.0D,1.0D,-1.0D, //2
                                        -1.0D,-1.0D,1.0D,1.0D, //3
                                        -1.0D,1.0D,-1.0D,-1.0D }; //4
        double[][] testData=new double[4][4];
        int dataIndex=0;
        for(int index=0;index<4;index++){
            for(int index1=0;index1<4;index1++){
                testData[index][index1]=binaryData[dataIndex];
                dataIndex++;
            }
        }
        binaryData=null;
        /* done setting up data, time to test the perceptron */
        double desiredOutput=1.0D;
        double learningRate=0.01D;
        double errorThreshold=0.01D;
        Perceptron perceptron=new Perceptron(desiredOutput,learningRate,learningRate,errorThreshold);
        perceptron.randomizeWeights(4,-5.0D,5.0D);
        perceptron.setInput(testData[0]);
        try{
            double actualOutput=perceptron.run();
            double startError=Math.abs(desiredOutput-actualOutput);
            for(int iteration=0;iteration<10;iteration++){
                perceptron.adjustWeights();
                actualOutput=perceptron.run();
                double newError=Math.abs(desiredOutput-actualOutput);
                if(newError>startError){
                    System.out.println("FAIL! newError ("+newError+") > startError ("+startError+")");
                } else{
                    System.out.println("PASS! newError ("+newError+") <= startError ("+startError+")");
                }
                startError=newError;
            }
        } catch(NeuronException nx){
            nx.printStackTrace();
        }
        //System.out.println("\nFinal perceptron");
        //System.out.println(perceptron.toString());
        System.out.println("Exit testPerceptron()");
    }    
    
    /* test the input parser, prints a ton of text - do not use regularly*/
    public static void testInputParser(){
        System.out.println("\nEnter testInputParser()");
        InputParser inputParser=new InputParser("C:\\School\\CS598\\Data\\train.dat");
        while(inputParser.moveNext()){
            System.out.println("\nRead input line with: ");
            System.out.println("expectedID="+inputParser.getExpectedID());
            System.out.println("expectedSeverity="+inputParser.getExpectedSeverity());
            System.out.println("expectedAction="+inputParser.getExpectedAction());
            double[][] inputWord=inputParser.getInputWord();
            for(int index1=0;index1<inputWord.length;index1++){
                StringBuffer line=new StringBuffer();
                line.append("inputWord["+index1+"]=");
                System.out.println(inputWord[index1]==null);
                for(int index2=0;index2<inputWord[index1].length;index2++){
                    line.append(String.valueOf(inputWord[index1][index2]));
                    line.append(" ");
                }
                System.out.println(line.toString());
            }
        }
        inputParser.close();
        System.out.println("Exit testInputParser()");
    }

    /* test Kohonen layer  */
    public static void testKohonenLayer(){
        System.out.println("\nEnter testKohonenLayer()");
        KohonenLayer layer=new KohonenLayer();
        double learningRate=0.5D;
        double errorThreshold=0.05D;
        double[] input={1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D};
        for(int neuronCount=0;neuronCount<10;neuronCount++){
            Perceptron neuron=new Perceptron(learningRate,learningRate,errorThreshold);
            neuron.randomizeWeights(input.length,-1.0D,1.0D);
            layer.insertNeuron(neuron);
        }
        double[] desiredOutput={1.0D,-1.0D,-1.0D,-1.0D,-1.0D,-1.0D,-1.0D,-1.0D,-1.0D,-1.0D};
        try{
            layer.setDesiredOutput(desiredOutput);
        } catch(LayerException lx){
            lx.printStackTrace();
        }
        layer.setInput(input);
        layer.run();
        System.out.println("\nInitial Layer=");
        System.out.println(layer.toString());
        double startError=layer.getError();
        double newError=layer.getError();
        int iteration=0;
        while((iteration++)<100&&(newError>0.0D)){
        //for(int iteration=0;iteration<100;iteration++){
            layer.train();
            layer.run();
            newError=layer.getError();
            if(newError>startError){
                System.out.println("FAIL! newError ("+newError+") > startError ("+startError+")");
            } else{
                System.out.println("PASS! newError ("+newError+") <= startError ("+startError+")");
            }
            startError=newError;
        }
        System.out.println("\nFinal Layer=");
        System.out.println(layer.toString());
        System.out.println("\nExit testKohonenLayer()");
    }    

    /* test BackPropagationLayer  */
    public static void testBackPropagationLayer(){
        System.out.println("\nEnter testBackPropagationLayer()");
        double learningRate=0.5D;
        double errorThreshold=0.5D;
        double maxLayerError=4.0D;
        double[] input={1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D,1.0D};
        double[] desiredOutput={1.0D,0.0D,0.0D,0.0D,0.0D,0.0D,0.0D,0.0D,0.0D,-1.0D};
        BackPropagationLayer layer=new BackPropagationLayer(desiredOutput.length,input.length,learningRate,learningRate,errorThreshold,-5.0D,5.0D);
        try{
            layer.setDesiredOutput(desiredOutput);
        } catch(LayerException lx){
            lx.printStackTrace();
        }
        layer.setInput(input);
        layer.run();
        double startError=layer.getError();
        layer.setDoneTraining(false);
        for(int iteration=0;iteration<100;iteration++){
            layer.train();
            layer.run();
            double newError=layer.getError();
            if(newError<maxLayerError){
                layer.setDoneTraining(true);
                maxLayerError*=0.75D;
            } else{
                if(newError==startError){
                    System.out.println("Improvement stopped!");
                    if(layer.getDoneTraining()==true){
                        System.out.println("Decreasing training rate..");
                        layer.adjustTrainingRate(0.75);
                    } else{
                        System.out.println("Increasing training rate..");
                        layer.adjustTrainingRate(1.25);
                    }
                } else if(newError>startError){
                    System.out.println("FAIL! newError ("+newError+") > startError ("+startError+")");
                } else{
                    System.out.println("PASS! newError ("+newError+") < startError ("+startError+")");
                }
            }
            startError=newError;
        }
        System.out.println("Layer=");
        System.out.println(layer.toString());
        System.out.println("Exit testBackPropagationLayer()");
    }    
    
    public static void testBackPropagationNetwork(){
        System.out.println("\nEnter testBackPropagationNetwork()");
        /* set up neural network */
        int inputLayerSize=21;
        int hiddenLayerSize=14;
        int outputLayerSize=7;
        int inputSize=3;
        double punishRate=0.5D;
        double rewardRate=0.0D;
        double errorThreshold=0.5D;
        double minWeight=-5.0D;
        double maxWeight=5.0D;
        BackPropagationNetwork network=new BackPropagationNetwork(inputLayerSize,hiddenLayerSize,outputLayerSize,inputSize,punishRate,rewardRate,errorThreshold,minWeight,maxWeight);
        /* read input */
        InputParser inputParser=new InputParser("C:\\School\\CS598\\Data\\train.dat");
        int counter=0;
        while(inputParser.moveNext()){
            double[][] inputWord=inputParser.getInputWord();
            System.out.println("input word size");
            for(int index=0;index<inputWord.length;index++){
                System.out.println("inputWord["+index+"]="+inputWord[index].length);
            }
            double[] input=new double[3];
            double[] desiredOutput=new double[outputLayerSize];
            for(int index=0;index<outputLayerSize;index++){
                desiredOutput[index]=-1.0D;
            }
            if(inputParser.getExpectedID()==1.0D){ //"code red" scan
                input[0]=1.0D;
                desiredOutput[0]=-1;
                desiredOutput[1]=-1;
                desiredOutput[2]=1;
                desiredOutput[3]=-1;
                desiredOutput[4]=1;
                desiredOutput[5]=-1;
                desiredOutput[6]=1;
            } else if(inputParser.getExpectedID()==2.0D){ //trojan horse scan
                input[0]=-1.0D;
                desiredOutput[0]=-1;
                desiredOutput[1]=1;
                desiredOutput[2]=-1;
                desiredOutput[3]=1;
                desiredOutput[4]=-1;
                desiredOutput[5]=1;
                desiredOutput[6]=-1;
            } else{ //ignore
                desiredOutput[0]=-1;
                desiredOutput[1]=-1;
                desiredOutput[2]=-1;
                desiredOutput[3]=-1;
                desiredOutput[4]=-1;
                desiredOutput[5]=-1;
                desiredOutput[6]=-1;                
            }
            if(inputParser.getExpectedSeverity()>0.0D){
                input[1]=1.0D;
            } else{
                input[1]=-1.0D;
            }
            if(inputParser.getExpectedAction()>0.0D){
                input[2]=1.0D;
            } else{
                input[2]=-1.0D;
            }
            network.setInput(input);
            network.setDesiredOutput(desiredOutput);
            network.run();
            double startError=network.getError();
            network.train(20);
            network.run();
            double newError=network.getError();
            if(newError==startError){
                System.out.println("Improvement stopped!");
            } else if(newError>startError){
                System.out.println("FAIL! newError ("+newError+") > startError ("+startError+")");
            } else{
                System.out.println("PASS! newError ("+newError+") < startError ("+startError+")");
            }
            double[] output=network.getOutput();
            System.out.println("\nResults for instance "+(counter++)+" : ");
            if((input[0]>0.0D&&output[0]>0.0D)||(input[0]<0.0D&&output[0]<0.0D)){
                System.out.println("expected id is correct :)");
            } else{
                System.out.println("expected id is incorrect :(");
            }
            if((input[1]>0.0D&&output[1]>0.0D)||(input[1]<0.0D&&output[1]<0.0D)){
                System.out.println("expected severity is correct :)");
            } else{
                System.out.println("expected severity is incorrect :(");
            }
            if((input[2]>0.0D&&output[2]>0.0D)||(input[2]<0.0D&&output[2]<0.0D)){
                System.out.println("expected action is correct :)");
            } else{
                System.out.println("expected action is incorrect :(");
            }
            System.out.println("raw error="+network.getError());
        }
        inputParser.close();
        System.out.println("\nExit testBackPropagationNetwork()");
    }
    
    /* test LamstarNetwork */
    public static void testNetworkVirusLamstarNetwork(){
        System.out.println("\nEnter testNetworkVirusLamstarNetwork()");
        NetworkVirusLamstarNetwork network=new NetworkVirusLamstarNetwork();
        /* read input */
        //InputParser inputParser=new InputParser("C:\\School\\CS598\\Data\\train-0100.dat");
        InputParser inputParser=new InputParser("C:\\School\\CS598\\Data\\mix.dat");
        //int counter=0;
        while(inputParser.moveNext()){
            double[][] inputWord=inputParser.getInputWord();
            //System.out.println("input word size");
            //for(int index=0;index<inputWord.length;index++){
            //    System.out.println("inputWord["+index+"]="+inputWord[index].length);
            //}
            /* determine the desired output */
            double[][] desiredOutput=new double[3][3];
            for(int index1=0;index1<3;index1++){
                for(int index2=0;index2<3;index2++){
                    desiredOutput[index1][index2]=-1.0D;
                }
            }
            desiredOutput[0][(int)inputParser.getExpectedID()]=1.0D;
            desiredOutput[1][(int)inputParser.getExpectedSeverity()]=1.0D;
            desiredOutput[2][(int)inputParser.getExpectedAction()]=1.0D;
            /* run the network */
            try{
                network.setInput(inputWord);
                network.setDesiredOutput(desiredOutput);
                network.run();
                double startError=network.getError();
                System.out.println("\nstartError="+startError);
                /* try training */
                network.train(50);
                network.run();
                double newError=network.getError();
                System.out.println("newError="+newError);
                double[][] actualOutput=network.getNetworkOutput();
                System.out.println("expected output");
                for(int index1=0;index1<3;index1++){
                    System.out.print("[ ");
                    for(int index2=0;index2<3;index2++){
                        System.out.print(desiredOutput[index1][index2]+" ");
                    }
                    System.out.println("]");
                }
                System.out.println("actual output");
                for(int index1=0;index1<actualOutput.length;index1++){
                    System.out.print("[ ");
                    for(int index2=0;index2<actualOutput[index1].length;index2++){
                        System.out.print(actualOutput[index1][index2]+" ");
                    }
                    System.out.println("]");
                }
                double[][] networkDesiredOutput=network.getDesiredOutput();
                System.out.println("networkDesiredOutput output");
                for(int index1=0;index1<networkDesiredOutput.length;index1++){
                    System.out.print("[ ");
                    for(int index2=0;index2<networkDesiredOutput[index1].length;index2++){
                        System.out.print(networkDesiredOutput[index1][index2]+" ");
                    }
                    System.out.println("]");
                }                
            } catch(NeuralNetworkException nnx){
                nnx.printStackTrace();
            } catch(LayerException lx){
                lx.printStackTrace();
            }
        //    double[] input=new double[3];
        //    double[] desiredOutput=new double[outputLayerSize];
        //    for(int index=0;index<outputLayerSize;index++){
        //        desiredOutput[index]=-1.0D;
        //    }
        //    if(inputParser.getExpectedID()==1.0D){ //"code red" scan
        //        input[0]=1.0D;
        //        desiredOutput[0]=-1;
        //        desiredOutput[1]=-1;
        //        desiredOutput[2]=1;
        //        desiredOutput[3]=-1;
        //        desiredOutput[4]=1;
        //        desiredOutput[5]=-1;
        //        desiredOutput[6]=1;
        //    } else if(inputParser.getExpectedID()==2.0D){ //trojan horse scan
        //        input[0]=-1.0D;
        //        desiredOutput[0]=-1;
        //        desiredOutput[1]=1;
        //        desiredOutput[2]=-1;
        //        desiredOutput[3]=1;
        //        desiredOutput[4]=-1;
        //        desiredOutput[5]=1;
        //        desiredOutput[6]=-1;
        //    } else{ //ignore
        //        desiredOutput[0]=-1;
        //        desiredOutput[1]=-1;
        //        desiredOutput[2]=-1;
        //        desiredOutput[3]=-1;
        //        desiredOutput[4]=-1;
        //        desiredOutput[5]=-1;
        //        desiredOutput[6]=-1;                
        //    }
        //    if(inputParser.getExpectedSeverity()>0.0D){
        //        input[1]=1.0D;
        //    } else{
        //        input[1]=-1.0D;
        //    }
        //    if(inputParser.getExpectedAction()>0.0D){
        //        input[2]=1.0D;
        //    } else{
        //        input[2]=-1.0D;
        //    }
        //    network.setInput(input);
        //    network.setDesiredOutput(desiredOutput);
        //    network.run();
        //    double startError=network.getError();
        //    network.train(20);
        //    network.run();
        //    double newError=network.getError();
        //    if(newError==startError){
        //        System.out.println("Improvement stopped!");
        //    } else if(newError>startError){
        //        System.out.println("FAIL! newError ("+newError+") > startError ("+startError+")");
        //    } else{
        //        System.out.println("PASS! newError ("+newError+") < startError ("+startError+")");
        //    }
        //    double[] output=network.getOutput();
        //    System.out.println("\nResults for instance "+(counter++)+" : ");
        //    if((input[0]>0.0D&&output[0]>0.0D)||(input[0]<0.0D&&output[0]<0.0D)){
        //        System.out.println("expected id is correct :)");
        //    } else{
        //        System.out.println("expected id is incorrect :(");
        //    }
        //    if((input[1]>0.0D&&output[1]>0.0D)||(input[1]<0.0D&&output[1]<0.0D)){
        //        System.out.println("expected severity is correct :)");
        //    } else{
        //        System.out.println("expected severity is incorrect :(");
        //    }
        //    if((input[2]>0.0D&&output[2]>0.0D)||(input[2]<0.0D&&output[2]<0.0D)){
        //        System.out.println("expected action is correct :)");
        //    } else{
        //        System.out.println("expected action is incorrect :(");
        //    }
        //    System.out.println("raw error="+network.getError());
        }
        inputParser.close();        
        System.out.println("\nExit testNetworkVirusLamstarNetwork()");
    }
    
    /** entry point for test routines
     * @param args not used
     */
    public static void main(String[] args){
        long startTime=System.currentTimeMillis();
        //for(int index=0;index<5;index++){
        //    testPerceptron();
        //}
        //testInputParser();
        //testKohonenLayer();
        //testBackPropagationLayer();
        //testBackPropagationNetwork();
        testNetworkVirusLamstarNetwork();
        System.out.println("\n\nAll tests completed in "+((System.currentTimeMillis()-startTime)/1000.0D)+" seconds");
        System.exit(0);
    }
}